/***************************************************************************
 * NAG.C -- Convert between PGN NAG value and ChessBase evaluation.
 *
 * Copyright (c)1993 Andy Duplain.
 *
 * Version      Date            Comments
 * =======      ====            ========
 * 1.0          01/02/94        Initial.
 ***************************************************************************/

#include "global.h"
#include "nag.h"

struct nag nags[MAX_NAG + 1] = {
    {0, 0},                            /* 0 */
    {1, VAR_EVAL},                     /* 1 */
    {2, VAR_EVAL},                     /* 2 */
    {5, VAR_EVAL},                     /* 3 */
    {6, VAR_EVAL},                     /* 4 */
    {3, VAR_EVAL},                     /* 5 */
    {4, VAR_EVAL},                     /* 6 */
    {15, VAR_POSEVAL},                 /* 7 */
    {15, VAR_POSEVAL},                 /* 8 */
    {0, 0},                            /* 9 */
    {4, VAR_POSEVAL},                  /* 10 */
    {4, VAR_POSEVAL},                  /* 11 */
    {4, VAR_POSEVAL},                  /* 12 */
    {5, VAR_POSEVAL},                  /* 13 */
    {3, VAR_POSEVAL},                  /* 14 */
    {6, VAR_POSEVAL},                  /* 15 */
    {2, VAR_POSEVAL},                  /* 16 */
    {7, VAR_POSEVAL},                  /* 17 */
    {1, VAR_POSEVAL},                  /* 18 */
    {8, VAR_POSEVAL},                  /* 19 */
    {1, VAR_POSEVAL},                  /* 20 */
    {8, VAR_POSEVAL},                  /* 21 */
    {0, 0},                            /* 22 */
    {0, 0},                            /* 23 */
    {0, 0},                            /* 24 */
    {0, 0},                            /* 25 */
    {0, 0},                            /* 26 */
    {0, 0},                            /* 27 */
    {0, 0},                            /* 28 */
    {0, 0},                            /* 29 */
    {9, VAR_EVAL},                     /* 30 */
    {9, VAR_EVAL},                     /* 31 */
    {9, VAR_EVAL},                     /* 32 */
    {9, VAR_EVAL},                     /* 33 */
    {9, VAR_EVAL},                     /* 34 */
    {9, VAR_EVAL},                     /* 35 */
    {12, VAR_POSEVAL},                 /* 36 */
    {12, VAR_POSEVAL},                 /* 37 */
    {12, VAR_POSEVAL},                 /* 38 */
    {12, VAR_POSEVAL},                 /* 39 */
    {13, VAR_POSEVAL},                 /* 40 */
    {13, VAR_POSEVAL},                 /* 41 */
    {0, 0},                            /* 42 */
    {0, 0},                            /* 43 */
    {10, VAR_POSEVAL},                 /* 44 */
    {10, VAR_POSEVAL},                 /* 45 */
    {10, VAR_POSEVAL},                 /* 46 */
    {10, VAR_POSEVAL},                 /* 47 */
    {0, 0},                            /* 48 */
    {0, 0},                            /* 49 */
    {0, 0},                            /* 50 */
    {0, 0},                            /* 51 */
    {0, 0},                            /* 52 */
    {0, 0},                            /* 53 */
    {0, 0},                            /* 54 */
    {0, 0},                            /* 55 */
    {0, 0},                            /* 56 */
    {0, 0},                            /* 57 */
    {0, 0},                            /* 58 */
    {0, 0},                            /* 59 */
    {0, 0},                            /* 60 */
    {0, 0},                            /* 61 */
    {0, 0},                            /* 62 */
    {0, 0},                            /* 63 */
    {0, 0},                            /* 64 */
    {0, 0},                            /* 65 */
    {0, 0},                            /* 66 */
    {0, 0},                            /* 67 */
    {0, 0},                            /* 68 */
    {0, 0},                            /* 69 */
    {0, 0},                            /* 70 */
    {0, 0},                            /* 71 */
    {0, 0},                            /* 72 */
    {0, 0},                            /* 73 */
    {0, 0},                            /* 74 */
    {0, 0},                            /* 75 */
    {0, 0},                            /* 76 */
    {0, 0},                            /* 77 */
    {0, 0},                            /* 78 */
    {0, 0},                            /* 79 */
    {0, 0},                            /* 80 */
    {0, 0},                            /* 81 */
    {0, 0},                            /* 82 */
    {0, 0},                            /* 83 */
    {0, 0},                            /* 84 */
    {0, 0},                            /* 85 */
    {0, 0},                            /* 86 */
    {0, 0},                            /* 87 */
    {0, 0},                            /* 88 */
    {0, 0},                            /* 89 */
    {0, 0},                            /* 90 */
    {0, 0},                            /* 91 */
    {0, 0},                            /* 92 */
    {0, 0},                            /* 93 */
    {0, 0},                            /* 94 */
    {0, 0},                            /* 95 */
    {0, 0},                            /* 96 */
    {0, 0},                            /* 97 */
    {0, 0},                            /* 98 */
    {0, 0},                            /* 99 */
    {0, 0},                            /* 100 */
    {0, 0},                            /* 101 */
    {0, 0},                            /* 102 */
    {0, 0},                            /* 103 */
    {0, 0},                            /* 104 */
    {0, 0},                            /* 105 */
    {0, 0},                            /* 106 */
    {0, 0},                            /* 107 */
    {0, 0},                            /* 108 */
    {0, 0},                            /* 109 */
    {0, 0},                            /* 110 */
    {0, 0},                            /* 111 */
    {0, 0},                            /* 112 */
    {0, 0},                            /* 113 */
    {0, 0},                            /* 114 */
    {0, 0},                            /* 115 */
    {0, 0},                            /* 116 */
    {0, 0},                            /* 117 */
    {0, 0},                            /* 118 */
    {0, 0},                            /* 119 */
    {0, 0},                            /* 120 */
    {0, 0},                            /* 121 */
    {0, 0},                            /* 122 */
    {0, 0},                            /* 123 */
    {0, 0},                            /* 124 */
    {0, 0},                            /* 125 */
    {0, 0},                            /* 126 */
    {0, 0},                            /* 127 */
    {0, 0},                            /* 128 */
    {0, 0},                            /* 129 */
    {11, VAR_POSEVAL},                 /* 130 */
    {11, VAR_POSEVAL},                 /* 131 */
    {11, VAR_POSEVAL},                 /* 132 */
    {11, VAR_POSEVAL},                 /* 133 */
    {11, VAR_POSEVAL},                 /* 134 */
    {11, VAR_POSEVAL},                 /* 135 */
    {14, VAR_POSEVAL},                 /* 136 */
    {14, VAR_POSEVAL},                 /* 137 */
    {14, VAR_POSEVAL},                 /* 138 */
    {14, VAR_POSEVAL}                  /* 139 */
};
